@@ -550,6 +550,88 @@ class Insight_Definition:
         print(url)
         return url
 
+    def add_admin(self, admin_email):
+
+        get_url = self._base_url_askdata+"/smartbot/share/principals?q="+admin_email
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": "Bearer" + " " + self._token
+        }
+
+        r = s.get(url=get_url, headers=headers)
+        r.raise_for_status()
+        if(r.json()!=[]):
+            body = {
+                "role": "ADMIN",
+                "userId": r.json()["id"]
+            }
+
+            url = self.smart_insight_url+"/insights/8a09ec79-8e1e-401a-bdec-70d33c9c1543/users"
+
+            s = requests.Session()
+            s.keep_alive = False
+            retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+            s.mount('https://', HTTPAdapter(max_retries=retries))
+
+            headers = {
+                "Content-Type": "application/json",
+                "Authorization": "Bearer" + " " + self._token
+            }
+            query_url = self.smart_insight_url + '/definitions/' + self.definition_id + '/components/'
+            logging.info("URL {}".format(query_url))
+            r = s.post(url=url, json=body, headers=headers)
+            r.raise_for_status()
+        else:
+            logging.error("EMAIL ERRATA: Inserire una email corrispondente ad un utente Askdata")
+
+
+    def add_follower(self, follower_email):
+
+        get_url = self._base_url_askdata+"/smartbot/share/principals?q="+follower_email
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": "Bearer" + " " + self._token
+        }
+
+        r = s.get(url=get_url, headers=headers)
+        r.raise_for_status()
+        if(r.json()!=[]):
+            body = {
+                "role": "FOLLOWER",
+                "userId": r.json()["id"]
+            }
+
+            url = self.smart_insight_url+"/insights/8a09ec79-8e1e-401a-bdec-70d33c9c1543/users"
+
+            s = requests.Session()
+            s.keep_alive = False
+            retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+            s.mount('https://', HTTPAdapter(max_retries=retries))
+
+            headers = {
+                "Content-Type": "application/json",
+                "Authorization": "Bearer" + " " + self._token
+            }
+            query_url = self.smart_insight_url + '/definitions/' + self.definition_id + '/components/'
+            logging.info("URL {}".format(query_url))
+            r = s.post(url=url, json=body, headers=headers)
+            r.raise_for_status()
+        else:
+            logging.error("EMAIL ERRATA: Inserire una email corrispondente ad un utente Askdata")
+
+
     def edit_card(self, icon="", name=""):
 
         if(icon==""):
